"use strict";
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("mocha");
const fdb = require("../lib");
const assert = require("assert");
const util_1 = require("./util");
(0, util_1.withEachDb)(db => describe('key value functionality', () => {
    const batchToStrUnprefix = (batch) => (batch.map(([k, v]) => [k.toString(), v.toString()]));
    const prefill = async () => {
        const _db = db.at(null, util_1.numXF, util_1.numXF);
        await _db.doTransaction(async (tn) => {
            // Originally I just filled 100 values, but getEstimatedRangeSize needs more.
            for (let i = 0; i < 1000; i++)
                tn.set(i, i);
        });
        return _db;
    };
    it('returns all values through getRange iteration', async () => {
        const _db = await prefill();
        await _db.doTransaction(async (tn) => {
            var _a, e_1, _b, _c;
            let i = 0;
            try {
                for (var _d = true, _e = __asyncValues(tn.getRange(0, 1000)), _f; _f = await _e.next(), _a = _f.done, !_a;) {
                    _c = _f.value;
                    _d = false;
                    try {
                        const [key, val] = _c;
                        assert.strictEqual(key, i);
                        assert.strictEqual(val, i);
                        i++;
                    }
                    finally {
                        _d = true;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) await _b.call(_e);
                }
                finally { if (e_1) throw e_1.error; }
            }
            assert.strictEqual(i, 1000);
        });
    });
    it('can correctly reverse the range query', async () => {
        const _db = await prefill();
        await _db.doTransaction(async (tn) => {
            var _a, e_2, _b, _c;
            let i = 1000;
            try {
                for (var _d = true, _e = __asyncValues(tn.getRange(0, 1000, { reverse: true })), _f; _f = await _e.next(), _a = _f.done, !_a;) {
                    _c = _f.value;
                    _d = false;
                    try {
                        const [key, val] = _c;
                        i--;
                        assert.strictEqual(key, i);
                        assert.strictEqual(val, i);
                    }
                    finally {
                        _d = true;
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) await _b.call(_e);
                }
                finally { if (e_2) throw e_2.error; }
            }
            assert.strictEqual(i, 0);
        });
    });
    it('returns all values through getRangeBatch', async () => {
        const _db = await prefill();
        await _db.doTransaction(async (tn) => {
            var _a, e_3, _b, _c;
            let i = 0;
            try {
                for (var _d = true, _e = __asyncValues(tn.getRangeBatch(0, 1000)), _f; _f = await _e.next(), _a = _f.done, !_a;) {
                    _c = _f.value;
                    _d = false;
                    try {
                        const batch = _c;
                        for (let k = 0; k < batch.length; k++) {
                            const [key, val] = batch[k];
                            assert.strictEqual(key, i);
                            assert.strictEqual(val, i);
                            i++;
                        }
                    }
                    finally {
                        _d = true;
                    }
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) await _b.call(_e);
                }
                finally { if (e_3) throw e_3.error; }
            }
            assert.strictEqual(i, 1000);
        });
    });
    it('supports raw string ranges against the root database', async () => {
        // Regression - https://github.com/josephg/node-foundationdb/pull/39
        // This regression requires that we run a naked query without a prefix,
        // which is difficult to do with the current API
        await db.getRoot().doTransaction(async (tn) => {
            var _a, e_4, _b, _c;
            try {
                for (var _d = true, _e = __asyncValues(tn.getRange('a', 'b')), _f; _f = await _e.next(), _a = _f.done, !_a;) {
                    _c = _f.value;
                    _d = false;
                    try {
                        const batch = _c;
                    }
                    finally {
                        _d = true;
                    }
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) await _b.call(_e);
                }
                finally { if (e_4) throw e_4.error; }
            }
        });
    });
    it('fetches tuple ranges using a prefix correctly', async () => {
        const _db = db.withKeyEncoding(fdb.tuple);
        await _db.set(['a\x00'], 'no');
        await _db.set(['a', 'b'], 'yes');
        assert.deepStrictEqual(await _db.getRangeAllStartsWith(['a']), [[['a', 'b'], Buffer.from('yes')]]);
    });
    it('clears tuple ranges using a prefix correctly', async () => {
        const _db = db.withKeyEncoding(fdb.tuple);
        await _db.set(['a\x00'], 'no');
        await _db.set(['a', 'b'], 'yes');
        await _db.clearRangeStartsWith('a');
        // That should have deleted yes, but not no.
        assert.equal(await _db.get(['a', 'b']), null);
        assert.deepStrictEqual(await _db.get(['a\x00']), Buffer.from('no'));
    });
    it('getRange without a specified end uses start as a prefix');
    it('calls getEstimatedRangeSize correctly', async () => {
        const _db = await prefill();
        const size = await _db.getEstimatedRangeSizeBytes(0, 1000);
        assert(typeof size === 'number');
        // assert(size > 0) // This might not be reliable because getEstimatedRangeSize could change its implementation.
    });
    it('getRangeSplitPoints splits the range', async () => {
        const _db = await prefill();
        const keys = await _db.getRangeSplitPoints(0, 1000, 3);
        assert(keys.length >= 2);
        keys.forEach(x => assert(typeof x === 'number'));
    });
    describe('selectors', () => {
        const data = [['a', 'A'], ['b', 'B'], ['c', 'C']];
        beforeEach(async () => {
            await db.doTransaction(async (tn) => {
                data.forEach(([k, v]) => tn.set(k, v));
            });
        });
        it('raw string range queries get [start,end)', async () => {
            const result = batchToStrUnprefix(await db.getRangeAll('a', 'c'));
            assert.deepEqual(result, data.slice(0, 2)); // 'a', 'b'.
        });
        it('returns [start, end) with firstGreaterThanEq selectors', async () => {
            const result = batchToStrUnprefix(await db.getRangeAll(fdb.keySelector.firstGreaterOrEqual('a'), fdb.keySelector.firstGreaterOrEqual('c')));
            assert.deepEqual(result, data.slice(0, 2)); // 'a', 'b'.
        });
        it('returns (start, end] with firstGreaterThan selectors', async () => {
            const result = batchToStrUnprefix(await db.getRangeAll(fdb.keySelector.firstGreaterThan('a'), fdb.keySelector.firstGreaterThan('c')));
            assert.deepEqual(result, data.slice(1)); // 'b', 'c'.
        });
    });
}));
//# sourceMappingURL=range.js.map